#include "tweet.h"
#include "tweets.h"
#include "twittero.h"
#include "twitteros.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define MAX_TWEET 141

typedef enum {
	TWITTEAR = 1,
	CUENTAS,
	CONSULTAS,
	SIN_USO,
	SALIR
} Opciones;

typedef enum {
	CONSULTAR_TWEETS = 1,
	CONSULTAR_TWITTEROS,
	BUSCAR_TWEETS
} Consultas;

typedef enum {
	INICIAR_SESION = 1,
	REGISTRARSE
} Cuentas;

void menu(const twittero * u, int * opcion, int * subopcion);
void leer_opcion(int * opcion, int * subopcion);
void vaciar_stdin();
void leer_linea(const char * prompt, char buf[]);
void registrarse(twitteros * users);
void buscar_tweet(tweets * tts);
void cleanuptwitteros(twitteros * users);
void cleanuptweets(tweets * tts);


void twittear(twittero * u, tweets * tts);
twittero * iniciar_sesion(twittero * u, twitteros * users);

int main()
{
	int opcion;
	int subopcion;

	tweets * tts;
	twitteros * users;
	twittero * u;

	/* Inicializamos */
	tts = new_tweets();
	if (tts == NULL) {
		fprintf(stderr, "Fatal Error: Error al reservar memoria\n");
		return(EXIT_FAILURE);
	}

	users = new_twitteros();
	if (users == NULL) {
		destroy_tweets(tts);
		fprintf(stderr, "Fatal Error: Error al reservar memoria\n");

		return(EXIT_FAILURE);
	}

	u = NULL;

	/* Fin inicializacion */


	menu(u, &opcion, &subopcion);

	while (opcion != SALIR) {

		switch (opcion) {

		case TWITTEAR:

			twittear(u, tts);

			break;

		case CUENTAS:

			switch (subopcion) {
			case INICIAR_SESION:

				u = iniciar_sesion(u, users);

				break;

			case REGISTRARSE:

				registrarse(users);

				break;

			default:
				printf("Opcion Incorrecta\n");

			}

			break;

		case CONSULTAS:

			switch (subopcion) {
			case CONSULTAR_TWEETS:
				print_tweets(tts);
				break;
			case CONSULTAR_TWITTEROS:
				print_twitteros(users);

				break;

			case BUSCAR_TWEETS:
				buscar_tweet(tts);

				break;

			default:
				printf("Opcion Incorrecta\n");
			}

			break;

		case SALIR:
			/* Nunca llega aqui */
			break;

		default:
			printf("Opcion Incorrecta\n");
		}


		menu(u, &opcion, &subopcion);
	}

	cleanuptweets(tts);
	cleanuptwitteros(users);


	printf("Adios\n");

	return(EXIT_SUCCESS);

}

/**
 * Funcion para mostrar el menu y leer la opcion escogida por el usuario
 * 
 * Entrada:
 *	El twittero con la sesión iniciada
 * 
 * Argumentos de salida:
 *	La opción y subopción escogidas por el usuario
 */
void menu(const twittero * u, int * opcion, int * subopcion)
{

	printf("******************************\n");
	printf("*     1.Twittear             *\n");
	printf("*     2.Cuentas              *\n");

	if (u == NULL) {
		printf("*        2.1 Iniciar Sesion  *\n");
	} else {
		printf("*        2.1 Cerrar Sesion   *\n");

	}
	printf("*        2.2 Registrarse     *\n");
	printf("*     3. Consutas            *\n");
	printf("*        3.1 Tweets          *\n");
	printf("*        3.2 Twitteros       *\n");
	printf("*        3.3 Buscar Tweets   *\n");
	printf("*     5. Salir               *\n");
	printf("******************************\n");
	printf("Elige una opcion > ");

	leer_opcion(opcion, subopcion);


}


/**
 * Función para leer la opcion del menu introducida por el usuario en formato
 * n.n o n si se trata de una opción principal. Si el valor es incorrecto, se
 * almacena un -1 en *opcion
 *  
 * Argumentos de salida:
 *	opcion, pasada por referencia, donde almacenar la opción escogida por el
 *	usuario.
 *	subopcion, pasada por referencia, donde almacenar la subopción escogida 
 *	por el usuario.
 */
void leer_opcion(int * opcion, int * subopcion)
{
	int control;
	
	control = scanf("%d.%d", opcion, subopcion);
	vaciar_stdin();


	if (control < 1 || ((*opcion == CUENTAS || *opcion == CONSULTAS) && control < 2)) {
		*opcion = -1;
	}

}

/**
 * Función para leer la entrada estándar hasta completar una línea
 * Extrae caracteres de stdin hasta encontrarse con un \n o un EOF
 */
void vaciar_stdin()
{
	int c;

	do {
		c = getchar();
	} while (c != '\n' && c != EOF);

}



/**
 * Función para leer una linea de 140 caracteres
 * Solicita al usuario una linea de texto de un máximo de 140 caracteres con el
 * prompt indicado en el primer argumento. La línea es almacenada en el array
 * de caracteres del segundo argumento sin el caracter de salto de línea+
 * 
 * Entrada:
 *    promp a mostrar para solicitar la línea y
 *    array de caracteres en los que almacenar la línea
 * */
void leer_linea(const char * prompt, char buf[])
{
	printf("%s (%d Caracteres): ", prompt, MAX_TWEET - 1);
	fgets(buf, MAX_TWEET, stdin);

	if (buf[strlen(buf) - 1] == '\n') {
		buf[strlen(buf) - 1] = '\0';
	}
}

/**
 * Función para twittear
 * Si el usuario u está con la sesión iniciada (no es NULL), se solicita el
 * texto del tweet al usuario. A continuación se almacena en la colección de
 * tweets de la aplicación así como en la propia del usuario
 * 
 * Entrada:
 *    Twittero actual con la sesion abierta (NULL si no hay ninguno) y
 *    Colección de tweets a actualizar
 * */

void twittear(twittero * u, tweets * tts)
{
	char texto[MAX_TWEET];
	tweet * t;

	if (u == NULL) {
		printf("Tiene que iniciar sesion antes\n");
		return;
	}

	leer_linea("¿Que sucede?", texto);

	t = new_tweet();
	if (t == NULL) {
		fprintf(stderr, "Fatal Error: Error al reservar memoria\n");
		return;
	}

	set_tweet_text(t, texto);

	if (add_tweet(tts, t) < 0) {
		fprintf(stderr, "Error al aniadir tweet a la coleccion\n");
		return;
	}

	add_twittero_tweet(u, t);


}

/**
 * Función para actualizar el usuario activo en twitter
 * Si el twittero pasado por argumento no es NULL, cierra la sesión, 
 * devolviendo NULL. Si por el contrario, u es NULL, solicita un alias al 
 * usuario, devolviendo el usuario encontrado en la colección de twitteros o
 * NULL si no lo encuentra
 * 
 * Entrada:
 *    Twittero actual con la sesion abierta (NULL si no hay ninguno) y
 *    Colección de twitteros para buscar
 * 
 * Salida: 
 *    Twittero con la sesión iniciada tras la ejecución de la función (NULL si
 *    no hay ninguno)
 */

twittero * iniciar_sesion(twittero * u, twitteros * users)
{
	char alias[MAX_TWEET];

	if (u != NULL) {
		printf("Adios (%s). Sesion cerrada correctamente\n", get_twittero_name(u));
		return NULL;
	}

	leer_linea("Introduce el alias", alias);

	u = get_twittero_by_alias(users, alias);

	if (u == NULL) {
		printf("Usuario no encontrado\n");
	} else {
		printf("Sesion iniciada correctamente\n");
	}

	return u;
}

/**
 * Función para darle de alta en twitter 
 * Solicita una Nombre y alias al usuario y lo almacena en la colección de
 * twitteros pasada por argumento, si el alias no estuviese repetido.
 * En cualquier caso, informa si la operación se realizó o no con éxito.
 *
 * Entrada:
 *    Colección de twitteros a actualizar
 */
void registrarse(twitteros * users)
{
	twittero * u;

	char nombre[MAX_TWEET];
	char alias[MAX_TWEET];

	printf("Introduce los datos:\n");
	leer_linea("    Nombre", nombre);
	leer_linea("    Alias", alias);

	u = new_twittero();

	if (u == NULL) {
		fprintf(stderr, "Fatal Error: Error al reservar memoria\n");
		return;
	}
	
	set_twittero_name(u, nombre);
	set_twittero_alias(u, alias);

	if (add_twittero(users, u) == NULL_ID) {
		printf("No se pudo aniadir el nuevo twittero: alias no disponible\n");
	} else {
		printf("Twittero aniadido correctamente\n");
	}
}

/**
 * Función para buscar la ocurrencia de una palabra en el conjunto de tweets
 * Solicita una palabra al usuario e imprime todos los tweets en los que se 
 * encuentre esa secuencia de caracteres
 *
 * Entrada:
 *    Colección de tweets a buscar
 */

void buscar_tweet(tweets * tts)
{
	char patron[MAX_TWEET];
	tweet * t;
	int i;

	leer_linea("Introduce la palabra", patron);

	for (i = 0; i < get_n_tweets(tts); i++) {
		t = get_tweet_i(tts, i);

		if (strstr(get_tweet_text(t), patron) != NULL) {
			print_tweet(t);
		}
	}

}

/**
 * Función para liberar recursos asociados a la colección de twitteros. 
 * En concreto libera la memoria de todos los twitteros almacenados así 
 * como la memoria propia de la colección
 *
 * Entrada:
 *    Colección de twitteros a liberar
 */

void cleanuptwitteros(twitteros * users) {
	int i;
	twittero * u;
	
	if(users == NULL){
		return;
	}
	
	
	for(i = 0; i < get_n_twitteros(users); i++){
		u = get_twittero_i(users, i);
		destroy_twittero(u);
	}
	
	destroy_twitteros(users);
}

/**
 * Funcion para liberar recursos asociados a la colección de tweets. En concreto
 * libera la memoria de todos los tweets almacenados así como la memoria propia
 * de la colección
 *
 * Entrada:
 *    Colección de tweets a liberar
 */
void cleanuptweets(tweets * tts) {
	int i;
	tweet * t;
	
	if(tts == NULL){
		return;
	}
	
	
	for(i = 0; i < get_n_tweets(tts); i++){
		t = get_tweet_i(tts, i);
		destroy_tweet(t);
	}
	
	destroy_tweets(tts);
}