/**
 * Fichero : tweet.c      Fecha:23/12/2016
 * Autores:                                   
 *                                            
 *  fichero con la implementacion del TAD tweet
 *                                            
 */

#include "tweet.h"
#include "const.h"

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

/*********************************************** Estructura del TAD tweet ***/
struct _tweet {
	long long int id;
	char * text;
	twittero * author;
};

/**
 * Funcion para crear una nueva estructura tweet 
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura tweet inicializado todo a vacio o NULL en caso de error
 */
tweet * new_tweet()
{
	tweet *t;

	t = (tweet *) malloc(sizeof(tweet));
	if (t == NULL) {
		printf("Error de reserva de memoria/n");
		return NULL;
	}

	t->id = NULL_ID;
	t->text = NULL;
	t->author = NULL;

	return t;
}

/**
 * Funcion para liberar la memoria reservada de la estructura tweet
 *
 * Entrada:
 *    Estructura tweet a liberar
 */
void destroy_tweet(tweet *t)
{
	assert(t);

	if (t->text != NULL) {
		free(t->text);
	}

	free(t);
}

/**
 * Funcion para establecer el identificador del tweet.
 *
 * Entrada:
 *    Estructura tweet e id del tweet
 *
 */
void set_tweet_id(tweet * t, long long int id)
{
	assert(t);

	t->id = id;

}

/**
 * Funcion para establecer el texto del tweet.
 *
 * OJO: Esta función debe hacer una copia de la cadena que se pasa
 *
 * Entrada:
 *    Estructura tweet y el texto del tweet
 *
 */
void set_tweet_text(tweet * t, char * text)
{
	char * nuevo_text;
	assert(t != NULL && text != NULL);

	/* Reservamos la longitud de la cadena mas 1 para el terminador nulo */
	nuevo_text = (char *) realloc(t->text, (strlen(text) + 1) * sizeof(char));

	if (nuevo_text == NULL) {
		return;
	}

	t->text = nuevo_text;

	strcpy(t->text, text);

}

/**
 * Funcion para establecer el autor del tweet.
 *
 * Entrada:
 *    Estructura tweet e autor del tweet
 *
 */
void set_tweet_author(tweet * t, twittero *u)
{
	assert(t);

	t->author = u;
}

/**
 * Funcion para recuperar el id de un tweet
 *
 * Entrada:
 *    Estructura del TAD tweet sobre el que se quiere recuperar el id
 *
 * Salida:
 *    El id del tweet
 */
long long int get_tweet_id(tweet * t)
{
	assert(t);

	return t->id;
}

/**
 * Funcion para recuperar el texto de un tweet
 *
 * Entrada:
 *    Estructura del TAD tweet sobre el que se quiere recuperar el texto
 *
 * Salida:
 *    El texto del tweet (es constante para que no se pueda modificar
 *        desde fuera)
 */
const char * get_tweet_text(tweet * t)
{
	assert(t);

	return t->text;
}

/**
 * Funcion para recuperar el autor de un tweet
 *
 * Entrada:
 *    Estructura del TAD tweet sobre el que se quiere recuperar el autor
 *
 * Salida:
 *    El autor del tweet 
 */
twittero * get_tweet_author(tweet * t)
{
	assert(t);

	return t->author;
}

/**
 * Funcion para imprimir un tweet por pantalla
 *
 * Entrada:
 *    Estructura TAD tweet a imprimir
 */
void print_tweet(tweet *t)
{
	if (t == NULL) {
		printf("NULL\n");
		return;
	}

	printf("ID: %lld\n", t->id);

	/*Descomentar cuando implementemos twittero, comentar para pasar test_tweet*/
	printf("@%s\n", get_twittero_alias(t->author));
	/*---------*/

	printf("%s\n\n", t->text);


}


