/**
 * Fichero : tweet.h      Fecha:23/12/2014
 * Autores:                                   
 *                                            
 *  fichero con la cabecera del TAD tweet
 *                                            
 */


#ifndef __TWEET_H_
#define __TWEET_H_


/*********************************************** Estructura del TAD tweet ***/
typedef struct _tweet tweet;

#include "twittero.h"

/**
 * Funcion para crear una nueva estructura tweet 
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura tweet inicializado todo a vacio o NULL en caso de error
 */
tweet * new_tweet();

/**
 * Funcion para liberar la memoria reservada de la estructura tweet
 *
 * Entrada:
 *    Estructura tweet a liberar
 */
void destroy_tweet(tweet *t);

/**
 * Funcion para establecer el identificador del tweet.
 *
 * Entrada:
 *    Estructura del TAD tweet y el identificador como un entero.
 *
 * Salida:
 */
void set_tweet_id(tweet * t, long long int id);

/**
 * Funcion para establecer el texto del tweet.
 *
 * OJO: Esta función debe hacer una copia de la cadena que se pasa
 *
 * Entrada:
 *    Estructura del TAD tweet y el texto del tweet.
 *
 * Salida:
 */
void set_tweet_text(tweet * t, char * text);

/**
 * Funcion para establecer el autor del tweet.
 *
 * Entrada:
 *    Estructura tweet y autor del tweet
 *
 */
void set_tweet_author(tweet *t, twittero *user);

/**
 * Funcion para recuperar el id de un tweet
 *
 * Entrada:
 *    Estructura del TAD tweet sobre el que se quiere recuperar el id
 *
 * Salida:
 *    El id del tweet
 */
long long int get_tweet_id(tweet * t);

/**
 * Funcion para recuperar el texto de un tweet
 *
 * Entrada:
 *    Estructura del TAD tweet sobre el que se quiere recuperar el texto
 *
 * Salida:
 *    El texto del tweet (es constante para que no se pueda modificar
 *        desde fuera)
 */
const char * get_tweet_text(tweet * t);

/**
 * Funcion para recuperar el autor de un tweet
 *
 * Entrada:
 *    Estructura del TAD tweet sobre el que se quiere recuperar el autor
 *
 * Salida:
 *    El autor del tweet
 */
twittero *get_tweet_author(tweet * t);

/**
 * Funcion para imprimir un tweet por pantalla
 *
 * Entrada:
 *    Estructura TAD tweet a imprimir
 */
void print_tweet(tweet *t);

#endif

