/**
 * Fichero : twittero.h      Fecha:23/12/2014
 * Autores:                                   
 *                                            
 *  fichero con la cabecera del TAD twittero
 *                                            
 */

#ifndef __TWITTERO_H_
#define __TWITTERO_H_

/*********************************************** Estructura del TAD twittero ***/
typedef struct _twittero twittero;

#include "tweet.h"
#include "const.h"

/**
 * Funcion para crear una nueva estructura twittero 
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura twittero inicializado todo a vacio o NULL en caso de error
 */
twittero * new_twittero();

/**
 * Funcion para liberar la memoria reservada de la estructura  TAD twittero
 *
 * Entrada:
 *    Estructura twittero a liberar
 */
void destroy_twittero(twittero * user);

/**
 * Funcion para asignar el id a un twittero.
 *
 * Entrada:
 *    Estructura twittero e id del twittero
 *
 */
void set_twittero_id(twittero * user, long long int id);

/**
 * Funcion para asignar el nombre a un twittero.
 *
 * Entrada:
 *    Estructura twittero y nombre a asignar
 *
 */
void set_twittero_name(twittero * user, const char * name);

/**
 * Funcion para asignar el alias a un twittero. No debe contener espacios
 *
 * Entrada:
 *    Estructura twittero y el alias a asignar
 *
 */
void set_twittero_alias(twittero * user, const char * alias);

/**
 * Funcion para recuperar el id de un twittero
 *
 * Entrada:
 *    Estructura twittero sobre el que se quiere recuperar el id
 *
 * Salida:
 *    El id del twittero
 */
long long int get_twittero_id(twittero * user);

/**
 * Funcion para recuperar el nombre de un twittero
 *
 * Entrada:
 *    Estructura twittero sobre el que se quiere recuperar el nombre
 *
 * Salida:
 *    El nombre del twittero (es constante para que no se pueda modificar
 *        desde fuera)
 */
const char * get_twittero_name(twittero * user);

/**
 * Funcion para recuperar el alias de un twittero
 *
 * Entrada:
 *    Estructura twittero sobre el que se quiere recuperar el alias
 *
 * Salida:
 *    El alias del twittero (es constante para que no se pueda modificar        
 *        desde fuera)
 */
const char * get_twittero_alias(twittero * user);

/**
 * Funcion para aniadir un tweet a un twittero. 
 * No se comprueban duplicados (i.e., que el mismo tweet sea aniadido dos veces)
 *
 * Entrada:
 *    El twittero y el tweet que se quiere aniadir.
 *
 * Salida:
 *    
 */
void add_twittero_tweet(twittero *user, tweet * t);

/**
 * Funcion que indica si un tweet es de un twittero
 *
 * Entrada:
 *    El twittero y el tweet que se quiere buscar.
 *
 * Salida:
 *  TRUE si a esta en user y FALSE en caso contrario
 *    
 */
BOOL has_twittero_tweet(twittero *user, tweet * t);

/**
 * Funcion para imprimir un twittero por pantalla
 *
 * Entrada:
 *    Estructura del TAD twittero a imprimir
 */
void print_twittero(twittero * user);

#endif
