/**
 * Fichero : twitteros.h      Fecha:23/12/2014
 * Autores:                                   
 *                                            
 *  fichero con la cabecera del TAD twitteros
 *                                            
 */

/*#include "twittero.h"*/

#ifndef __TWITTEROS_H_
#define __TWITTEROS_H_

#include "twittero.h"

/*********************************************** Estructura del TAD twitteros ***/
typedef struct _twitteros twitteros;

/**
 * Funcion para crear una nueva estructura twittero 
 *
 * Entrada:
 *
 * Salida:
 *    Una estructura twitteros inicializado todo a vacio o NULL en caso de error
 */
twitteros * new_twitteros();

/**
 * Funcion para liberar la memoria reservada de la estructura twitteros y todos los
 *   twitteros que estan en el array.
 *
 * Entrada:
 *    Estructura twitteros a liberar
 */
void destroy_twitteros(twitteros *users);

/**
 * Funcion para aniadir un twittero ya creado al array. No se pueden
 * repetir los alias ni ids.
 *
 * Entrada:
 *    Estructura del TAD twitteros sobre el que se quiere aniadir un twittero y
 *    puntero al twittero a aniadir. Si el twittero tinen ID_NULO, se le
 *    asignara el ultimo_id+1
 *
 * Salida:
 *    El id del twittero aniadido o NULL_ID en caso de error 
 */
long long int add_twittero(twitteros * users, twittero * t);

/**
 * Funcion para recuperar el numero de twitteros del array
 *
 * Entrada:
 *    Estructura del TAD twitteros sobre el que se quiere recuperar su tamanio
 *
 * Salida:
 *    El numero de twitteros
 */
int get_n_twitteros(twitteros * users);

/**
 * Funcion para recuperar el twittero n-esimo de la lista
 *
 * Entrada:
 *    Estructura twitteros e indice del twittero a recuperar (indexado sobre 0)
 *
 * Salida:
 *    Un puntero al twittero o NULL si hay un error
 */
twittero * get_twittero_i(twitteros *users, int index);


/**
 * Funcion para recuperar un twittero dado su código.
 *
 * Entrada:
 *    Colección de twitteros y el código del twittero a recuperar.
 *
 * Salida:
 *    Un puntero al twittero o NULL si no se encuentra.
 */
twittero * get_twittero_by_id(twitteros *users, int id);

/**
 * Funcion para recuperar un twittero dado su alias.
 *
 * Entrada:
 *    Colección de twitteros y el alias del twittero a recuperar.
 *
 * Salida:
 *    Un puntero al twittero o NULL si no se encuentra.
 */
twittero * get_twittero_by_alias(twitteros *users, const char *alias);

/**
 * Funcion para imprimir la lista de twitteros por pantalla
 *
 * Entrada:
 *    Estructura TAD twittero a imprimir
 */
void print_twitteros(twitteros *users); 

#endif

